@@ -43,6 +43,7 @@ from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
 from solfege.mpd import mpdutils
+from solfege.mpd.duration import Duration
 from solfege.mpd.rat import Rat
 
 _test_mode = False
@@ -84,7 +85,7 @@ def is_uri(filename):
 class LessonfileException(Exception):
     pass
 
-class MusicObjectException(mpd.MpdException):
+class MusicObjectException(LessonfileException):
     pass
 
 class LessonfileParseException(Exception):
@@ -161,6 +162,8 @@ keywords = (
     'rhythmtapping',
     'rhythmtapping2',
     'idproperty',
+    'rhythmdictation',
+    'rhythmdictation2',
     #
     'harmonic',
     'melodic',
@@ -179,6 +182,7 @@ keywords = (
     'accidentals',
     'key',
     'semitones',
+    'atonal',
     'play',
     'show',
     # elembuilder:
@@ -345,7 +349,7 @@ class _Header(dict):
                            ('title', ''),
                            ('description', ''),
                            ('musicformat', 'normal'),
-                           ('random_transpose', True),
+                           ('random_transpose', (True,)),
                            ('labelformat', 'normal'),
                            ('fillnum', 1),
                            ('filldir', 'horiz'),
@@ -372,6 +376,111 @@ class _Header(dict):
             return self[name]
         return istr("")
 
+class Bar(list):
+    class Item(object):
+        """
+        An object representing either a note or a rest.
+        """
+        def __init__(self, isnote, duration, tie):
+            assert isinstance(duration, Duration)
+            self.m_isnote = isnote
+            self.m_duration = duration
+            self.m_tie = tie
+    class BadDuration(Exception):
+        pass
+    class FullException(Exception):
+        pass
+    stepwidth = 30
+    def __init__(self, keysig):
+        assert isinstance(keysig, Rat)
+        self.m_keysig = keysig
+    def _do_add(self, isnote, n):
+        if not isinstance(n, Duration):
+            if not isinstance(n, unicode):
+                raise self.BadDuration(n)
+            n = Duration.new_from_string(n)
+        if self.duration() + n.get_rat_value() > self.m_keysig:
+            raise self.FullException()
+        self.append(self.Item(isnote, n, False))
+    def isfull(self):
+        return self.duration() == self.m_keysig
+    def can_append(self, item):
+        """
+        Return True if the bar is not full and there is space to add item.
+        """
+        return (self.duration()
+                + item.m_duration.get_rat_value() <= self.m_keysig)
+    def add_note(self, n):
+        self._do_add(True, n)
+    def add_rest(self, n):
+        self._do_add(False, n)
+    def duration(self):
+        r = Rat(0)
+        for item in self:
+            r += item.m_duration.get_rat_value()
+        return r
+    def do_layout(self, show_timesig):
+        self.m_show_timesig = show_timesig
+        if show_timesig:
+            xpos = self.stepwidth
+        else:
+            xpos = self.stepwidth / 2
+        for i in self:
+            i.m_xpos = xpos
+            xpos += 30
+        #If the bar is full, then the width of the bar is the m_xpos
+        #of the last note + Bar.stepwidth
+        if self.isfull():
+            self.m_width = self[-1].m_xpos + Bar.stepwidth
+        self.m_width = xpos + int((self.m_keysig - self.duration()) * 4 * Bar.stepwidth)
+    def _get_timelist(self):
+        """
+        A timelist is a list of list:
+        [isnote, duration] of the types [bool, Rat]
+        """
+        v = []
+        tie = False
+        for item in self:
+            if tie:
+                v[-1][1] += item.m_duration.get_rat_value()
+            else:
+                v.append([item.m_isnote, item.m_duration.get_rat_value()])
+            tie = item.m_tie
+        return v
+
+
+class RhythmStaff(list):
+    """
+    A staff is a list of Bars
+    A object that know how long rhythm to enter, and will
+    know of all time signature changes and bar lines.
+
+    A question in the lesson file could be written like this:
+    \time 3/4 c4 c8 c8 c4 | \time 3/8 c8 c8 c16 c16
+    or:
+    4 8 8 4 | 8 8 16 16
+
+    We need code that can transform the former to the last format so
+    that we can compare the real music to the rhythm entered by
+    the user.
+    """
+    def do_layout(self):
+        timesig = None
+        xpos = 0
+        for bar in self:
+            bar.m_xpos = xpos
+            bar.do_layout(timesig != bar.m_keysig)
+            timesig = bar.m_keysig
+            xpos += bar.m_width
+    def get_timelist(self):
+        v = []
+        for bar in self:
+            v.extend(bar._get_timelist())
+        return v
+    def append(self, bar):
+        if not isinstance(bar, Bar):
+            raise TypeError("RhythmStaff.append: only Bar allowed")
+        list.append(self, bar)
 
 class MusicBaseClass(object):
     """
@@ -399,15 +508,25 @@ class MpdParsable(MusicBaseClass):
     def play(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         instrument = lessonfile_ref.get_instruments(question, 1)
-        if len(instrument) == 2:
-            utils.play_music(self.get_mpd_music_string(lessonfile_ref),
-                lessonfile_ref.get_tempo(),
-                instrument[0],
-                instrument[1])
-        else:
-            utils.play_music3(self.get_mpd_music_string(lessonfile_ref),
-                lessonfile_ref.get_tempo(),
-                instrument)
+        # We have to call get_mpd_string outside the try: block because
+        # the exceptions raised by get_mpd_string has indexes relative to
+        # self.m_musicdata, and utils.play_music will raise exceptions
+        # with indexes relative to the complete mpd code (including the
+        # \staff codes etc.
+        mpdstring = self.get_mpd_music_string(lessonfile_ref)
+        try:
+            if len(instrument) == 2:
+                utils.play_music(mpdstring,
+                    lessonfile_ref.get_tempo(),
+                    instrument[0],
+                    instrument[1])
+            else:
+                utils.play_music3(mpdstring,
+                    lessonfile_ref.get_tempo(),
+                    instrument)
+        except mpd.MpdException, e:
+            self.complete_to_musicdata_coords(lessonfile_ref, e)
+            raise
         if _test_mode:
             return self.get_mpd_music_string(lessonfile_ref)
     def play_slowly(self, lessonfile_ref, question):
@@ -421,47 +540,156 @@ class MpdParsable(MusicBaseClass):
             return self.get_mpd_music_string(lessonfile_ref)
     def get_err_context(self, exception, lessonfile_ref):
         """
+        This functions expect exception.m_lineno, .m_linepos1 and .m_linepos2
+        to be set relative to the m_musicdata of the music object,
+        and not relative to the complete mpd string with added \staff{ } etc.
+
         Return a twoline string showing what caused the exception.
         """
-        # I had to comment out the assert statement below, becuase the rhythmtapping
-        # exercise will analyze the m_musicdata string directly and not the data from
-        # get_mpd_music_string. And then the error can be on line 0.
-        #assert exception.m_lineno != 0, "This nevers happend"
-        first = exception.m_linepos1
-        last = exception.m_linepos2
-        if self.m_musicdata.count("\n") + 1 < exception.m_lineno:
-            return "(FIXME: we need better error reporting)\nBad input to the music object of type %s causes the\nfollowing generated music code:" % self.__class__.__name__.lower() + "\n" +  self.get_mpd_music_string(lessonfile_ref)
-        return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno-1],
-            " " * first + "^" * (last - first)))
+        if getattr(exception, 'm_obj_lineno', False) is not False:
+            return "\n".join((self.m_musicdata.split("\n")[exception.m_obj_lineno],
+            " " * exception.m_linepos1 + "^" * (exception.m_linepos2 - exception.m_linepos1)))
+        return self._mpd_err_context(exception, lessonfile_ref)
+    def _mpd_err_context(self, exception, lessonfile_ref):
+        m = self.get_mpd_music_string(lessonfile_ref)
+        v = m.split("\n")
+        v = textwrap.wrap("Bad input to the music object of type %s made the parser fail to parse the following generated music code:" % self.__class__.__name__.lower(), 50) + v[:exception.m_lineno+1] + [" " * exception.m_linepos1 + "^" * (exception.m_linepos2 - exception.m_linepos1)] + v[exception.m_lineno+1:]
+        return "\n".join(v)
+    def complete_to_musicdata_coords(self, lessonfile_ref, exception):
+        """
+        The string from m_musicdata should always begin
+        on line 1 (0-indexed) of column 0.
 
+        This function modifies m_lineno, m_linepos1 and m_linepos2 so that
+        get_err_context can create an ok error message.
+        """
+        # These two lines are just for extra checking...
+        assert getattr(exception, '_coords_done', False) == False
+        exception._coords_done = True
+        if (exception.m_lineno > self.m_musicdata.count("\n") + 1
+            or exception.m_lineno == 0):
+            # Do not set m_obj_lineno since the error happened in the
+            # added music code, not in the m_musicdata code.
+            return
+        exception.m_obj_lineno = exception.m_lineno - 1
+    def get_rhythmstaff(self, lessonfile_ref):
+        """
+        Create and return a RhythmStaff showing the rhythm of the music
+        object.
+        """
+        staff = RhythmStaff()
+        timesig = None
+        bar_content = Rat(0, 1)
+        # the current note/rest-duration
+        duration = Duration(4, 0)
+        for toc_type, toc in mpd.parser.Lexer(
+                self.get_mpd_music_string(lessonfile_ref)):
+            if toc_type in (mpd.parser.Lexer.NOTE, mpd.parser.Lexer.REST):
+                if timesig is None:
+                    # very first note and no timesig parsed
+                    staff.append(Bar(Rat(4, 4)))
+                    timesig = Rat(4, 4)
+                if staff[-1].isfull():
+                    staff.append(Bar(timesig))
+                if toc.m_duration:
+                    duration = toc.m_duration
+                staff[-1].append(Bar.Item(toc_type == mpd.parser.Lexer.NOTE,
+                                duration, False))
+            elif toc_type == mpd.parser.Lexer.TIME:
+                timesig = Rat(toc.m_num, toc.m_den)
+                staff.append(Bar(timesig))
+            elif toc_type == u'~':
+                staff[-1][-1].m_tie = True
+            elif toc_type == u'|':
+                if staff[-1].m_keysig != bar_content:
+                    print "FIXME: barlines not ok."
+        return staff
+            
+ 
 
 class MpdDisplayable(MpdParsable):
     pass
 
+
 class MpdTransposable(MpdDisplayable):
-    pass
+    def get_first_pitch(self):
+        lexer = mpd.parser.Lexer(self.m_musicdata)
+        for toc, toc_data in lexer:
+            if toc == mpd.parser.Lexer.NOTE:
+                return toc_data.m_pitch
+        # We will display the first 3 non-empty lines
+        v = [x.strip() for x in self.m_musicdata.split("\n") if x]
+        v = v[:3]
+        raise MusicObjectException("%s\n%s" % 
+            ("get_first_pitch() could not find a musical pitch in the music data:",
+             "\n".join(v)))
+    def get_musicdata_transposed(self, lessonfile_ref):
+        """
+        Return m_musicdata, but transposed relative to 
+        lesonfile_ref.m_transpose. All pitches identified as Lexer.NOTE
+        are transposed.
+        """
+        assert lessonfile_ref.m_transpose
+        try:
+            lex = mpd.parser.Lexer(self.m_musicdata)
+            tokens = list(mpd.parser.Lexer(self.m_musicdata))
+        except mpd.InvalidNotenameException, e:
+            self.complete_to_musicdata_coords(lessonfile_ref, e)
+            raise
+        for toc, toc_data in tokens:
+            if isinstance(toc_data, mpd.requests.MusicRequest):
+                toc_data.m_pitch.transpose_by_musicalpitch(lessonfile_ref.m_transpose)
+        return mpd.parser.Lexer.to_string(tokens)
+    def relative_transpose_musicdata(self, lessonfile_ref):
+        """
+        Also remove octave changes from the first tone.
+        """
+        assert lessonfile_ref.m_transpose
+        tokens = list(mpd.parser.Lexer(self.m_musicdata))
+        first = True
+        for toc, toc_data in tokens:
+            if isinstance(toc_data, mpd.requests.MusicRequest):
+                orig_octave = toc_data.m_pitch.m_octave_i
+                n = toc_data.m_pitch.transpose_by_musicalpitch(lessonfile_ref.m_transpose)
+                n.m_octave_i = orig_octave
+                if not first:
+                    n.m_octave_i = orig_octave
+                else:
+                    n.m_octave_i = 0
+                    first = False
+                toc_data.m_pitch = n
+        return mpd.parser.Lexer.to_string(tokens)
+
 
 class ChordCommon(MpdTransposable):
     pass
 
 class Chord(ChordCommon):
     def get_lilypond_code(self, lessonfile_ref):
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            music = r"  { <%s> }" % self.get_musicdata_transposed(lessonfile_ref)
+        else:
+            music = r"\transpose c' %s{ <%s> }" % (
+                lessonfile_ref.m_transpose.get_octave_notename(),
+                self.m_musicdata)
         return r"\score{ "\
-               r" \transpose c' %s{ <%s> }"\
+               r" %s"\
                r" \layout { "\
                r"  ragged-last = ##t "\
                r'  \context { \Staff \remove "Time_signature_engraver" } '\
                r" }"\
-               r"}"  % (
-            lessonfile_ref.m_transpose.get_octave_notename(),
-            self.m_musicdata)
+               r"}"  % music
     def get_lilypond_code_first_note(self, lessonfile_ref):
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return r"{ %s }" % self.get_musicdata_transposed(lessonfile_ref).split()[0]
         return r"\transpose c' %s{ %s }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
             self.m_musicdata.split()[0])
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return "\\staff{<\n%s\n>}" % self.get_musicdata_transposed(lessonfile_ref)
+        if lessonfile_ref.header.random_transpose[0]:
             return "\\staff\\transpose %s{<\n%s\n>}" \
                 % (lessonfile_ref.m_transpose.get_octave_notename(),
                    self.m_musicdata)
@@ -482,12 +710,12 @@ class Chord(ChordCommon):
         # like chord("c'2 e").
         notenames = [self.strip_trailing_digits(n) for n in self.m_musicdata.split()]
         try:
-            if not lessonfile_ref.header.random_transpose:
+            if not lessonfile_ref.header.random_transpose[0]:
                 return [mpd.MusicalPitch.new_from_notename(n).get_octave_notename() for n in notenames]
             else:
                 return [mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename() for n in notenames]
         except mpd.InvalidNotenameException, e:
-            e.m_lineno, e.m_linepos1, e.m_linepos2 = mpd.parser.validate_only_notenames(self.m_musicdata)
+            e.m_obj_lineno, e.m_linepos1, e.m_linepos2 = mpd.parser.validate_only_notenames(self.m_musicdata)
             raise
     def get_music_as_notename_string(self, lessonfile_ref):
         return " ".join(self.get_music_as_notename_list(lessonfile_ref))
@@ -542,89 +770,105 @@ class Chord(ChordCommon):
             t3.notelen_time(4)
         t3.note(4, mpd.notename_to_int(nlist[-1]))
         soundcard.synth.play_track(t1, t2, t3)
-    def get_err_context(self, exception, lessonfile_ref):
-        return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
-            " " * exception.m_linepos1 + "^" * (exception.m_linepos2 - exception.m_linepos1)))
 
 class VoiceCommon(MpdTransposable):
-    def get_first_pitch(self):
-        a, b = mpdutils.find_possible_first_note(self.m_musicdata)
-        return self.m_musicdata[a:b]
+    pass
 
 class Voice(VoiceCommon):
     def get_lilypond_code(self, lessonfile_ref):
-        return r"\transpose c' %s{ %s }" % (
-            lessonfile_ref.m_transpose.get_octave_notename(),
-            self.m_musicdata)
+        assert isinstance(lessonfile_ref, LessonfileCommon)
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return r"{ %s }" % self.get_musicdata_transposed(lessonfile_ref)
+        elif lessonfile_ref.header.random_transpose[0]:
+            return r"\transpose c' %s{ %s }" % (
+                lessonfile_ref.m_transpose.get_octave_notename(),
+                self.m_musicdata)
+        return r"{ %s }" % self.m_musicdata
     def get_lilypond_code_first_note(self, lessonfile_ref):
-        return r"\score{" \
-               r" \new Staff<< "\
-               r"  \new Voice\transpose c' %s{ \cadenzaOn %s }"\
-               r"  \new Voice{ \hideNotes %s } "\
-               r" >>"\
-               r" \layout { ragged-last = ##t } " \
-               r"}" % (
-            lessonfile_ref.m_transpose.get_octave_notename(),
-            self.get_first_pitch(),
-            self.m_musicdata,
-            )
+        assert isinstance(lessonfile_ref, LessonfileCommon)
+        s = r"\score{" \
+            r" \new Staff<<"\
+            r" \new Voice%s{ \cadenzaOn %s }"\
+            r" \new Voice{ \hideNotes %s }"\
+            r" >>"\
+            r" \layout { ragged-last = ##t }" \
+            r" }"
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return s % (
+                "",
+                self.get_first_pitch().transpose_by_musicalpitch(
+                    lessonfile_ref.m_transpose).get_octave_notename(),
+                self.m_musicdata,
+                )
+        elif lessonfile_ref.header.random_transpose[0]:
+            return s % (
+                "\\transpose c' %s" % lessonfile_ref.m_transpose.get_octave_notename(),
+                self.get_first_pitch().get_octave_notename(),
+                self.m_musicdata,
+                )
+        return s % ("", self.get_first_pitch().get_octave_notename(), self.m_musicdata)
     def get_mpd_music_string(self, lessonfile_ref):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return "\\staff{\n%s\n}" % self.get_musicdata_transposed(lessonfile_ref)
+        if lessonfile_ref.header.random_transpose[0]:
             return "\\staff\\transpose %s{\n%s\n}" \
                 % (lessonfile_ref.m_transpose.get_octave_notename(),
                    self.m_musicdata)
         return "\\staff{\n%s\n}" % self.m_musicdata
 
 class Rvoice(VoiceCommon):
-    def get_err_context(self, exception, lessonfile_ref):
-        """
-        Return a twoline string showing what caused the exception.
-        """
-        if exception.m_lineno == 0:
-            # If there is an error in the first pitch, this will cause the
-            # mpd parser to raise and exception on the \relative XXX line,
-            # and this is on line 0.
-            a, b = mpdutils.find_possible_first_note(self.m_musicdata)
-            line = utils.string_get_line_at(self.m_musicdata, a)
-            a, b = mpdutils.find_possible_first_note(line)
-            return "\n".join((line, " " * (a) + "^" * (b - a)))
-        else:
-            return VoiceCommon.get_err_context(self, exception, lessonfile_ref)
     def get_mpd_music_string(self, lessonfile_ref):
+        """
+        Since get_mpd_music_string is used by error handling, it can not
+        raise any exceptions. If m_musicdata is bad, then it should provide
+        a string anyway, that the parser can choke on.
+        """
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        a, b = mpdutils.find_possible_first_note(self.m_musicdata)
-        if (a, b) != (None, None):
-            first_pitch = self.m_musicdata[a:b]
-        else:
-            first_pitch = None
-        if lessonfile_ref.header.random_transpose:
-            transpose_str = r"\transpose %s" % lessonfile_ref.m_transpose.get_octave_notename()
-        else:
-            transpose_str = ""
-        if not first_pitch:
-            # Returning there is mostly theoretical. Why would anyone
-            # have a music object with no notes, only commands like
-            # \clef, \staff etc?
-            return "\\staff%s{\n%s\n}"\
-               % (transpose_str, self.m_musicdata)
-        new_musicdata = self.m_musicdata[:a] + first_pitch.rstrip(",'") + self.m_musicdata[b:]
-        return "\\staff%s\\relative %s{\n%s\n}" \
-          % (transpose_str, first_pitch, new_musicdata)
+        if lessonfile_ref.header.random_transpose[0] == 'atonal':
+            return "\\staff\\relative %s{\n%s\n}" % (
+                self.get_first_pitch().transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename(),
+                self.relative_transpose_musicdata(lessonfile_ref))
+        lex = mpd.parser.Lexer(self.m_musicdata)
+        try:
+            pitch = self.get_first_pitch()
+        except mpd.InvalidNotenameException, e:
+            errpitch = self.m_musicdata.split("\n")[e.m_lineno][e.m_linepos1:e.m_linepos2]
+            if lessonfile_ref.header.random_transpose[0]:
+                return "\\staff\\transpose %s\\relative %s{ %s }" % (
+                    lessonfile_ref.m_transpose.get_octave_notename(),
+                    errpitch,#self.get_first_pitch().get_octave_notename(),
+                    lex.m_string
+                    )
+            # no translation
+            return "\\staff\\relative %s{\n%s\n}" \
+              % (errpitch, lex.m_string)
+            
+        pitch.m_octave_i = 0
+        lex.set_first_pitch(pitch)
+        if lessonfile_ref.header.random_transpose[0]:
+            return "\\staff\\transpose %s\\relative %s{ %s }" % (
+                lessonfile_ref.m_transpose.get_octave_notename(),
+                self.get_first_pitch().get_octave_notename(),
+                lex.m_string
+                )
+        # no translation
+        return "\\staff\\relative %s{\n%s\n}" \
+          % (self.get_first_pitch().get_octave_notename(), lex.m_string)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\transpose c' %s\relative c{ %s }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
             self.m_musicdata)
     def get_lilypond_code_first_note(self, lessonfile_ref):
         return r"\score{" \
-               r" \new Staff<< "\
-               r"  \new Voice\transpose c' %s\relative c{ \cadenzaOn %s }" \
-               r"  \new Voice{ \hideNotes %s } "\
-               r" >> "\
-               r" \layout { ragged-last = ##t } " \
-               r"}" % (
+               r" \new Staff<<"\
+               r" \new Voice\transpose c' %s\relative c{ \cadenzaOn %s }" \
+               r" \new Voice{ \hideNotes %s }"\
+               r" >>"\
+               r" \layout { ragged-last = ##t }" \
+               r" }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
-            self.get_first_pitch(),
+            self.get_first_pitch().get_octave_notename(),
             self.m_musicdata,
             )
 
@@ -650,7 +894,7 @@ class Satb(ChordCommon):
                 "\\addvoice{ \\stemDown <%s> }\n" \
                 "\\staff{ \key %s\\clef bass \\stemUp <%s>}\n"\
                 "\\addvoice{ \\stemDown <%s>}" % (k, v[0], v[1], k, v[2], v[3])
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0]:
             music = music.replace(r"\staff",
                       r"\staff\transpose %s" % lessonfile_ref.m_transpose.get_octave_notename())
             music = music.replace(r"\addvoice",
@@ -663,7 +907,7 @@ class Satb(ChordCommon):
         for x in 0, 1:
             s = voices[x].strip().split(" ")
             for n in s:
-                if lessonfile_ref.header.random_transpose:
+                if lessonfile_ref.header.random_transpose[0]:
                     n = mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename()
                 if cfg.get_string('user/sex') == 'female':
                     track.note(4, mpd.notename_to_int(n))
@@ -672,7 +916,7 @@ class Satb(ChordCommon):
         for x in 2, 3:
             s = voices[x].strip().split(" ")
             for n in s:
-                if lessonfile_ref.header.random_transpose:
+                if lessonfile_ref.header.random_transpose[0]:
                     n = mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename()
                 if cfg.get_string('user/sex') == 'male':
                     track.note(4, mpd.notename_to_int(n))
@@ -709,7 +953,11 @@ class PercBaseClass(MpdParsable):
     def get_mpd_music_string(self, lessonfile_ref):
         return "\\staff{\n%s\n}" % self.m_musicdata
     def _gen_track(self, lessonfile_ref, question):
-        score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
+        try:
+            score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
+        except mpd.MpdException, e:
+            self.complete_to_musicdata_coords(lessonfile_ref, e)
+            raise
         track = score.get_midi_events_as_percussion()[0]
         track.set_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.prepend_bpm(lessonfile_ref.get_tempo()[0],
@@ -781,7 +1029,7 @@ class Mma(MusicBaseClass):
         f = open(infile, "w")
         if self.m_groove:
             print >> f, "Groove %s" % self.m_groove
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0]:
             print >> f, "Transpose %i" % (lessonfile_ref.m_transpose.semitone_pitch() - mpd.MusicalPitch.new_from_notename("c'").semitone_pitch())
         f.write(self.m_musicdata)
         f.close()
@@ -838,9 +1086,9 @@ class CSound(MusicBaseClass):
         soundcard.play_mediafile('wav', os.path.join(self.temp_dir, "out.wav"))
 
 
-class Music(MpdDisplayable):
+class Music(MpdTransposable):
     def get_mpd_music_string(self, lessonfile_ref):
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0]:
             s = self.m_musicdata.replace(r'\staff',
                r'\staff\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
             s = s.replace(r'\addvoice',
@@ -853,7 +1101,7 @@ class Music(MpdDisplayable):
         """
         first = exception.m_linepos1
         last = exception.m_linepos2
-        if lessonfile_ref.header.random_transpose:
+        if lessonfile_ref.header.random_transpose[0]:
             s = self.m_musicdata.replace(r'\staff',
                r'\staff\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
             s = s.replace(r'\addvoice',
@@ -865,6 +1113,11 @@ class Music(MpdDisplayable):
         last = exception.m_linepos2 - i
         return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
             " " * first + "^" * (last - first)))
+    def complete_to_musicdata_coords(self, lessonfile_ref, exception):
+        # These two lines are just for extra checking...
+        assert getattr(exception, '_coords_done', False) == False
+        exception._coords_done = True
+        pass
 
 
 class Music3(Music):
@@ -926,7 +1179,14 @@ class LessonfileCommon(object):
                 # The following line is for backward compatibility
                 question.music = Music(question.music)
         self.m_random = xrandom.Random(range(len(self.m_questions)))
-        if self.header.random_transpose == True:
+        # We have to make random_transpose a list, since this make
+        # simplifies if statements in lessonfile.py
+        # Also we cannot define the 'yes' and 'no' lesson file keywords
+        # to be lists since it is used a a normal boolean for other
+        # variables.
+        if self.header.random_transpose in (True, False):
+            self.header.random_transpose = [self.header.random_transpose,]
+        if self.header.random_transpose[0] == True:
             self.header.random_transpose = ['key', -5, 5]
         # Backward compatability to handle old style
         # random_transpose = -4, 5 FIXMECOMPAT
@@ -954,7 +1214,7 @@ class QuestionsLessonfile(LessonfileCommon):
                 self.m_no_random_idx
             except:
                 self.m_no_random_idx = 0
-            self.header.random_transpose = False
+            self.header.random_transpose[0] = False
 
         count = 0
         available_question_idx = []
@@ -980,12 +1240,12 @@ class QuestionsLessonfile(LessonfileCommon):
                     self._idx = self.m_random.random_by_selection(available_question_idx)
                 else:
                     self._idx = random.choice(available_question_idx)
-            if self.header.random_transpose:
+            if self.header.random_transpose[0]:
                 self.m_transpose = self.find_random_transpose()
             if count == 10:
                 break
             if self.m_prev_question == self.get_music() \
-                and (len(self.m_questions) > 1 or self.header.random_transpose):
+                and (len(self.m_questions) > 1 or self.header.random_transpose[0]):
                 continue
             break
         try:
@@ -1004,9 +1264,9 @@ class QuestionsLessonfile(LessonfileCommon):
             key = self.m_questions[self._idx]['key']
         else:
             key = "c \major"
-        if self.header.random_transpose == True:
+        if self.header.random_transpose[0] == True:
             self.header.random_transpose = ['key', -5, 5]
-        if self.header.random_transpose[0] == 'semitones':
+        if self.header.random_transpose[0] in ('semitones', 'atonal'):
             retval = self.semitone_find_random_transpose()
             if random.randint(0, 1):
                 retval.enharmonic_flip()
@@ -1018,7 +1278,7 @@ class QuestionsLessonfile(LessonfileCommon):
         Called to find random transposition in "semitone" mode.
         Create and return a random MusicalPitch representing this transposition.
         """
-        assert self.header.random_transpose[0] == 'semitones'
+        assert self.header.random_transpose[0] in ('semitones', 'atonal')
         return mpd.MusicalPitch().randomize(
               mpd.transpose_notename("c'", self.header.random_transpose[1]),
               mpd.transpose_notename("c'", self.header.random_transpose[2]))
@@ -1259,7 +1519,7 @@ class TestSupport(object):
         assert self.m_test_idx < len(self.m_test_questions)
         self.m_test_idx += 1
         self._idx = self.m_test_questions[self.m_test_idx]
-        if self.header.random_transpose:
+        if self.header.random_transpose[0]:
             old = self.m_transpose
             # try really hard not to get the same tonika:
             for x in range(100):
