@@ -17,28 +17,8 @@
 
 from __future__ import absolute_import
 
-import logging.handlers
-
-from solfege import optionparser
-
 import solfege
 
-opt_parser = optionparser.SolfegeOptionParser()
-options, args = opt_parser.parse_args()
-
-if options.debug:
-    # We import all exercise modules here even though it is not necessary
-    # since the modules are loaded one by one when needed. But by importing
-    # all here, we catch SyntaxErrors at once.
-    from solfege.exercises import *
-
-    handler = logging.StreamHandler()
-    logging.getLogger().addHandler(handler)
-    logging.getLogger().setLevel(logging.DEBUG)
-else:
-    handler = logging.handlers.MemoryHandler(1)
-    logging.getLogger().addHandler(handler)
-
 
 import webbrowser
 import textwrap
@@ -55,66 +35,14 @@ except ValueError:
 import sys
 import traceback
 import os
-import time
 import urllib
 import shutil
 
 from solfege import winlang
-from solfege import runtime
 from solfege import buildinfo
-from solfege.exceptiondialog import ExceptionDialog
-
-solfege_copyright = u"Copyright © 1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
-
-warranty = """
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
-
-from solfege import osutils
-from solfege import application
-
-if options.version:
-    if buildinfo.is_release():
-        rev_info = ""
-    else:
-        rev_info = u"\n    " + u"\n    ".join(buildinfo.get_bzr_revision_info_list())
-    print (u"""GNU Solfege %s%s
-This is free software. It is covered by the GNU General Public License,
-and you are welcome to change it and/or distribute copies of it under
-certain conditions. Invoke as `solfege --warranty` for more information.
-
-%s
-        """ % (buildinfo.VERSION_STRING,
-        rev_info,
-        solfege_copyright)).encode(sys.getfilesystemencoding(), 'replace')
-    sys.exit()
-
-if options.warranty:
-    print "GNU Solfege %s" % buildinfo.VERSION_STRING
-    print solfege_copyright.encode(sys.getfilesystemencoding(), 'replace')
-    print warranty
-    sys.exit()
-
-# silent, GNOME, be silent!
-#sys.argv.append('--disable-sound')
-runtime.init(options)
+from solfege.esel import FrontPage, TestsView, SearchView
 
 import gtk
-# We need to set a uri hook that does nothing, since ubuntu 9.04 rc give
-# GConf errors and GtkWarnings without it.
-gtk.link_button_set_uri_hook(lambda a, b: None)
-
 from solfege import utils
 from solfege import i18n
 
@@ -142,25 +70,8 @@ class SplashWin(gtk.Window):
         while gtk.events_pending():
             gtk.main_iteration(0)
 
-if not options.no_splash:
-    splash_win = SplashWin()
-    time.sleep(0.1)
-    gtk.gdk.flush()
-    while gtk.events_pending():
-        gtk.main_iteration(0)
-else:
-    splash_win = None
-
-if splash_win:
-    splash_win.show_progress(_("Importing application modules"))
-
-from solfege import tracebackwindow
-# redirect error messages to a window that will popup if
-# something bad happens.
-
-sys.stderr = tracebackwindow.TracebackWindow(options.show_gtk_warnings)
-
 from solfege.configwindow import ConfigWindow
+from solfege.profilemanager import ChangeProfileDialog
 from solfege.mpd.musicdisplayer import MusicDisplayer
 from solfege import gu
 from solfege import cfg
@@ -168,6 +79,7 @@ from solfege import stock
 from solfege import lessonfile
 from solfege import lessonfilegui
 from solfege import abstract
+from solfege import statistics
 
 
 from solfege import frontpage
@@ -175,7 +87,6 @@ from solfege import fpeditor
 from solfege.trainingsetdlg import TrainingSetDialog
 from solfege.practisesheetdlg import PractiseSheetDialog
 from solfege import filesystem
-from solfege import statistics
 
 class MusicViewerWindow(gtk.Dialog):
     def __init__(self):
@@ -191,325 +102,6 @@ class MusicViewerWindow(gtk.Dialog):
         fontsize = cfg.get_int('config/feta_font_size=20')
         self.g_music_displayer.display(music, fontsize)
 
-class SelectWinBase(gtk.ScrolledWindow):
-    """
-    Base class for the classes that display the front page and the
-    page where we select exerises or tests.
-    """
-    def __init__(self):
-        gtk.ScrolledWindow.__init__(self)
-        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self._on_focus_in_blocked = False
-        self.m_min_width = 400
-        self.m_min_height = 300
-        self.set_size_request(self.m_min_width, self.m_min_height)
-        self.m_linkbuttons = []
-        self.g_searchentry = None
-    def on_focus_in(self, w, event):
-        """
-        Set the vadjustment so that the window will scroll to make the button
-        that has the focus visible in the scrolled window.
-        """
-        if self._on_focus_in_blocked:
-            return
-        a = w.get_allocation()
-        adj = self.get_vadjustment()
-        if a.y + a.height > adj.value + adj.page_size:
-            if w.get_data('last'):
-                adj.value = adj.upper - adj.page_size
-            else:
-                adj.value = a.y - adj.page_size + a.height
-        elif a.y < adj.value:
-            # If the widget is the first row of the first category, then...
-            if w.get_data('first'):
-                adj.value = 0.0
-            else:
-                adj.value = a.y
-    def on_key_press_event(self, mainwin, event):
-        if event.type == gtk.gdk.KEY_PRESS and event.state == 0:
-            adj = self.get_vadjustment()
-            if event.keyval in (gtk.keysyms.End, gtk.keysyms.KP_End):
-                if self.m_linkbuttons:
-                    self.m_linkbuttons[-1].grab_focus()
-                return True
-            elif event.keyval in (gtk.keysyms.Home, gtk.keysyms.KP_Home):
-                if self.g_searchentry:
-                    self.get_vadjustment().value = 0.0
-                    self.g_searchentry.grab_focus()
-                else:
-                    self.m_linkbuttons[0].grab_focus()
-                return True
-            if event.keyval in (gtk.keysyms.Page_Down, gtk.keysyms.KP_Page_Down):
-                # First we find the button that now has the keyboard focus
-                for idx, btn in enumerate(self.m_linkbuttons):
-                    if btn.is_focus():
-                        break
-                # Then we find a button that is approximately adj.page_increment
-                # down and grabs that buttons focus.
-                for to_btn in self.m_linkbuttons[idx:]:
-                    if to_btn.get_allocation().y - adj.value > adj.page_size:
-                        self._on_focus_in_blocked = True
-                        to_btn.grab_focus()
-                        self._on_focus_in_blocked = False
-                        return True
-                self.m_linkbuttons[-1].grab_focus()
-                return True
-            if event.keyval in (gtk.keysyms.Page_Up, gtk.keysyms.KP_Page_Up):
-                for idx, btn in enumerate(self.m_linkbuttons):
-                    if btn.is_focus():
-                        break
-                for to_btn in self.m_linkbuttons:
-                    if to_btn.get_allocation().y > adj.value - adj.page_increment:
-                        self._on_focus_in_blocked = True
-                        to_btn.grab_focus()
-                        self._on_focus_in_blocked = False
-                        return True
-    def adjust_scrolledwin_size(self):
-        # We do set_size_request on the view to make the window so wide
-        # that we avoid a horizontal scroll bar.
-        w = self.g_box.size_request()[0]
-        # It should be possible to check if the vscrollbar is visible, and
-        # only reserve space for it. But I cannot get that to works. I find
-        # that self.get_vscrollbar().props.visible is updated only part
-        # of the time.
-        w += self.get_vscrollbar().size_request()[0]
-        # FIXME I don't understand why we have to add the extra
-        # two pixels to avoid the horizontal scrollbar in all cases.
-        w += self.style_get_property("scrollbar-spacing") + 2
-        if w < self.m_min_width:
-            w = self.m_min_width
-        self.set_size_request(w, self.m_min_height)
-
-
-class ExerciseView(SelectWinBase):
-    max_exercise_label_width = int(gtk.gdk.screen_width() * 0.90)
-    def __init__(self):
-        SelectWinBase.__init__(self)
-        app = solfege.app
-        self.g_box = gtk.VBox()
-        self.g_box.set_border_width(gu.hig.SPACE_MEDIUM)
-        self.add_with_viewport(self.g_box)
-        self.g_searchbox = gtk.HBox()
-        self.g_box.pack_start(self.g_searchbox, False, padding=gu.hig.SPACE_MEDIUM)
-        self.g_searchentry = gtk.Entry()
-        self.g_searchbox.pack_start(self.g_searchentry, True)
-        self.g_searchentry.connect('activate', self.on_search)
-        gu.bButton(self.g_searchbox, _("Search"), callback=self.on_search, expand=False)
-        self.show_all()
-    def _display_data(self, page, display_only_tests=False,
-            is_search_result=False, show_topics=False):
-        """
-        display_only_tests should be True when we are browsing available tests.
-        This will validate statistics for each lesson and display the test result.
-        """
-        if not is_search_result:
-            self.m_page = page
-        try:
-            self.g_hbox.destroy()
-        except AttributeError:
-            pass
-        self.g_hbox = gu.bHBox(self.g_box)
-        self.g_hbox.set_spacing(gu.hig.SPACE_MEDIUM)
-        label = None
-        for column in page:
-            sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-            assert isinstance(column, frontpage.Column)
-            first = True
-            vbox = gtk.VBox()
-            vbox.set_spacing(gu.hig.SPACE_MEDIUM)
-            self.g_hbox.pack_start(vbox, False)
-            for sect_id, linklist in enumerate(column):
-                if isinstance(linklist, frontpage.Paragraph):
-                    label = gtk.Label(linklist.m_text)
-                    label.set_line_wrap(True)
-                    label.set_alignment(0.0, 0.5)
-                    vbox.pack_start(label, False)
-                    continue
-                assert isinstance(linklist, frontpage.LinkList)
-                if (display_only_tests
-                    and not frontpage._TreeCommon.tests_in_sub(linklist)):
-                        continue
-                linkbox = gtk.VBox()
-                vbox.pack_start(linkbox, False)
-                heading = gtk.Label("<b>%s</b>" % _no_xgettext(linklist.m_name))
-                heading.set_alignment(0.0, 0.5)
-                heading.set_use_markup(True)
-                linkbox.pack_start(heading, False)
-                for idx, link in enumerate(linklist):
-                    if isinstance(self, TestsView) and not frontpage._TreeCommon.tests_in_sub(link):
-                        continue
-                    if isinstance(link, frontpage.Page):
-                        label = gu.ClickableLabel(_no_xgettext(link.m_name))
-                        label.connect('clicked', self.on_page_link_clicked, link)
-                    else:
-                        assert isinstance(link, unicode), type(link)
-                        if display_only_tests:
-                            solfege.db.validate_stored_statistics(link)
-                        try:
-                            labeltxt = lessonfile.infocache.get(link, 'title')
-                            label = gu.ClickableLabel(_no_xgettext(labeltxt))
-                            if show_topics:
-                                topic = solfege.app.m_frontpage_data.get_topic_of_lesson_file(link)
-                                if topic and topic not in labeltxt:
-                                    label.add_heading(topic)
-                            label.connect('clicked', self.on_link_clicked, link)
-                        except lessonfile.InfoCache.FileNotFound:
-                            label = gu.ClickableLabel(_(u"«%s» was not found") % link)
-                            label.make_warning()
-                        except lessonfile.InfoCache.FileNotLessonfile:
-                            label = gu.ClickableLabel(_(u"Failed to parse «%s»") % link)
-                            label.make_warning()
-                    if first:
-                        label.set_data('first', True)
-                        first = False
-                    w = label.size_request()[0]
-                    if w > self.max_exercise_label_width:
-                        w = self.max_exercise_label_width
-                        if isinstance(link, unicode):
-                            txt = _(lessonfile.infocache.get(link, "title"))
-                        else:
-                            txt = link.m_name
-                        label.set_tooltip_text(txt)
-                        label.set_size_request(w / len(page), -1)
-                    self.m_linkbuttons.append(label)
-                    label.connect('focus-in-event', self.on_focus_in)
-                    if display_only_tests:
-                        box = gtk.HBox()
-                        box.pack_start(label)
-                        if isinstance(link, unicode):
-                            passed, result = solfege.db.get_test_status(link)
-                            if passed == True:
-                                box.pack_start(gtk.Label(_("passed, %.1f%%") % (result * 100)))
-                            if passed == False:
-                                box.pack_start(gtk.Label(_("failed, %.1f%%") % (result * 100)))
-                            # do nothing if passed == None since 
-                        linkbox.pack_start(box, True)
-                    else:
-                        linkbox.pack_start(label, True)
-                    sizegroup.add_widget(label)
-            if label:
-                label.set_data('last', True)
-        self.g_hbox.show_all()
-        self.adjust_scrolledwin_size()
-        self.get_vadjustment().value = 0.0
-    def on_page_link_clicked(self, btn, link):
-        self.g_searchbox.hide()
-        self.display_data(link)
-    def on_link_clicked(self, w, filename):
-        solfege.app.practise_lessonfile(filename)
-    def display_search_result(self, searchfor, result, result_C, display_only_tests=False):
-        self._display_data(
-          frontpage.Page(u'',
-            frontpage.Column([
-              frontpage.LinkList(
-            _("Search results for “%s”:" % self.g_searchentry.get_text()),
-              result),
-              frontpage.LinkList(
-            _("C-locale search results for “%s”:" % self.g_searchentry.get_text()),
-              result_C),
-              ])))
-    def on_end_practise(self, w=None):
-        pass
-    def _search(self, substring, C_locale, only_tests):
-        """
-        substring - the string to search for
-        C_locale - True if we should search in the untranslated titles
-        only_tests - True if we should only return exercises that have
-                     tests
-        """
-        logging.debug("search: '%s'" % substring)
-        match_func = {
-            False: lambda filename: substring in _no_xgettext(lessonfile.infocache.get(filename, 'title')).lower(),
-            True: lambda filename: substring in lessonfile.infocache.get(filename, 'title').lower()
-        }[C_locale]
-        test_filter = {
-            False: lambda filename: True,
-            True: lambda filename: lessonfile.infocache.get(filename, 'test')
-        }[C_locale]
-        page = frontpage.Page(listitems=frontpage.Column())
-        cur_topic = None
-        # the last topic appended to the page
-        last_topic = None 
-        found = set()
-        for child in self.m_page.iterate_flattened():
-            if isinstance(child, frontpage.LinkList):
-                cur_topic = child
-            if isinstance(child, unicode):
-                try:
-                    if (match_func(child) and test_filter(child)) and child not in found:
-                        if cur_topic != last_topic:
-                            page[0].append(frontpage.LinkList(cur_topic.m_name))
-                            last_topic = cur_topic
-                        found.add(child)
-                        page[0][-1].append(child)
-                except lessonfile.infocache.InfoCacheException:
-                    # Ignore missing lesson files and files with errors
-                    pass
-        return page
-
-
-class FrontPage(ExerciseView):
-    def display_data(self, data, display_only_tests=False,
-            is_search_result=False, show_topics=False):
-        self._display_data(data, display_only_tests, is_search_result,
-                           show_topics)
-        self.g_searchentry.grab_focus()
-    def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
-        logging.debug("FrontPage.on_search '%s'" % search_for)
-        page = self._search(search_for, False, False)
-        page_C = self._search(search_for, True, False)
-        page_C.m_name = _('Search untranslated lesson titles')
-        if not page_C.is_empty():
-            page[0].append(frontpage.LinkList(_('Too few matches?'),
-                listitems=[page_C]))
-        self.display_data(page, is_search_result=True)
-
-
-class TestsView(ExerciseView):
-    def on_link_clicked(self, w, filename):
-        solfege.app.test_lessonfile(filename)
-    def display_data(self, data=None, is_search_result=False, show_topics=False):
-        self._display_data(data, True, is_search_result, show_topics)
-        self.g_searchentry.grab_focus()
-    def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
-        page = self._search(search_for, False, True)
-        page_C = self._search(search_for, True, True)
-        page_C.m_name = u'Search exercises without translating them'
-        if not page_C.is_empty():
-            page[0].append(frontpage.LinkList(_('Too few matches?'),
-                listitems=[page_C]))
-        self.display_data(page, is_search_result=True)
-
-
-class SearchView(ExerciseView):
-    def __init__(self):
-        ExerciseView.__init__(self)
-        page = """FileHeader(1, 
-        Page(u'', [
-         Column([
-          Paragraph(u'Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.'),
-         ]),
-        ])
-        )"""
-        p = frontpage.parse_tree(page)
-        self.display_data(frontpage.parse_tree(page))
-        self.g_searchentry.show()
-    def display_data(self, data):
-        self._display_data(data)
-        self.g_searchentry.grab_focus()
-    def on_search(self, *button):
-        search_for = self.g_searchentry.get_text().lower()
-        lessonfile.infocache.update_modified_files()
-        self.display_search_result(self.g_searchentry.get_text(), [
-            filename for filename in lessonfile.infocache._data
-            if search_for in _(lessonfile.infocache.get(filename, 'title')).lower()
-        ], [
-            filename for filename in lessonfile.infocache._data
-            if search_for in lessonfile.infocache.get(filename, 'title').lower()
-        ])
-    
 
 class MainWin(gtk.Window, cfg.ConfigUtils):
     default_front_page = os.path.join(lessonfile.exercises_dir, 'learningtree.txt')
@@ -546,8 +138,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.main_box = gtk.VBox()
         self.main_box.show()
         self._vbox.pack_start(self.main_box)
-        self.on_frontpage_changed()
-        self.display_frontpage()
     def get_view(self):
         """
         Return the view that is currently visible.
@@ -583,11 +173,11 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         Change to a different frontpage file.
         """
         self.set_string('app/frontpage', filename)
-        self.on_frontpage_changed()
-    def on_frontpage_changed(self, *v):
+        self.load_frontpage()
+    def load_frontpage(self):
         """
-        Load the frontpage data from the file specified by the config
-        variable "app/frontpage"
+        Load the front page file set in the config database into
+        solfege.app.m_frontpage_data
         """
         filename = self.get_string("app/frontpage")
         if filename == self.debug_front_page and not solfege.app.m_options.debug:
@@ -598,14 +188,14 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         try:
             solfege.app.m_frontpage_data = frontpage.load_tree(filename)
         except Exception, e:
-            if splash_win:
-                splash_win.hide()
+            if solfege.splash_win:
+                solfege.splash_win.hide()
             solfege.app.m_frontpage_data = frontpage.load_tree(self.default_front_page)
             self.set_string('app/frontpage', self.default_front_page)
             gu.dialog_ok(_("Loading front page '%s' failed. Using default page." % filename),
                 secondary_text = "\n".join(traceback.format_exception(*sys.exc_info())))
-            if splash_win:
-                splash_win.show()
+            if solfege.splash_win:
+                solfege.splash_win.show()
         self.display_frontpage()
     def setup_menu(self):
         self.m_action_groups['Exit'].add_actions([
@@ -623,6 +213,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             self.new_training_set_editor),
           ('EditPractiseSheet', None, _(u'Ear Training Test Pri_ntout…'), None, None,
             self.new_practisesheet_editor),
+          ('ProfileManager', None, _("Profile _Manager"), None, None,
+            self.open_profile_manager),
           ('OpenPreferencesWindow', 'gtk-preferences', None, '<ctrl>F12', None,
             self.open_preferences_window),
           ('HelpMenu', None, _('_Help')),
@@ -675,6 +267,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         old_dir = None
         s = "<menubar name='Menubar'><menu action='FileMenu'><menu action='FrontPagesMenu'>"
         for fn in frontpage.get_front_pages_list(solfege.app.m_options.debug):
+            if solfege.splash_win:
+                solfege.splash_win.show_progress(fn)
             if not frontpage.may_be_frontpage(fn):
                 continue
             cur_dir = os.path.split(fn)[0]
@@ -742,28 +336,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 webbrowser.open_new("http://www.solfege.org?%s" % params)
             except Exception, e:
                 self.display_error_message2(_("Error opening web browser"), str(e))
-    def display_exception_message(self, exception, lessonfile=None):
-        """Call this function only inside an except clause."""
-        sourcefile, lineno, func, code = traceback.extract_tb(sys.exc_info()[2])[0]
-        # We can replace characters because we will only display the
-        # file name, not open the file.
-        sourcefile = sourcefile.decode(sys.getfilesystemencoding(), 'replace')
-        m = ExceptionDialog(exception)
-        if lessonfile:
-            m.add_text(_("Please check the lesson file %s.") % lessonfile)
-        if sourcefile:
-            m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
-        if 'm_nonwrapped_text' in dir(exception):
-            m.add_nonwrapped_text(exception.m_nonwrapped_text)
-        m.run()
-        m.destroy()
     def display_error_message2(self, text, secondary_text):
         """
         This is the new version of display_error_message, and it will
         eventually replace the old.
         """
-        if splash_win and splash_win.props.visible:
-            splash_win.hide()
+        if solfege.splash_win and solfege.splash_win.props.visible:
+            solfege.splash_win.hide()
             reshow_splash = True
         else:
             reshow_splash = False
@@ -778,12 +357,12 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         m.run()
         m.destroy()
         if reshow_splash:
-            splash_win.show()
+            solfege.splash_win.show()
             while gtk.events_pending():
                 gtk.main_iteration(0)
     def display_error_message(self, msg, title=None, secondary_text=None):
-        if splash_win and splash_win.props.visible:
-            splash_win.hide()
+        if solfege.splash_win and solfege.splash_win.props.visible:
+            solfege.splash_win.hide()
             reshow_splash = True
         else:
             reshow_splash = False
@@ -799,7 +378,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         m.run()
         m.destroy()
         if reshow_splash:
-            splash_win.show()
+            solfege.splash_win.show()
             while gtk.events_pending():
                 gtk.main_iteration(0)
     def show_path_info(self, w):
@@ -864,21 +443,20 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         fpeditor.Editor.edit_file(self.get_string("app/frontpage"))
     def post_constructor(self):
-        global splash_win
         self.m_frontpage_merge_id = None
         self.create_frontpage_menu()
         self.g_ui_manager.add_ui_from_file("help-menu.xml")
         if solfege.app.m_sound_init_exception is not None:
-            if splash_win:
-                splash_win.destroy()
-                splash_win = None
+            if solfege.splash_win:
+                solfege.splash_win.destroy()
+                solfege.splash_win = None
             solfege.app.display_sound_init_error_message(solfege.app.m_sound_init_exception)
         # MIGRATION 3.9.0
         if sys.platform == "win32" \
             and os.path.exists(os.path.join(filesystem.get_home_dir(), "lessonfiles")) \
             and not os.path.exists(filesystem.user_lessonfiles()):
-                if splash_win:
-                    splash_win.hide()
+                if solfege.splash_win:
+                    solfege.splash_win.hide()
                 do_move = gu.dialog_yesno(_('In Solfege 3.9.0, the location where Solfege look for lesson files you have created was changed. The files has to be moved from "%(old)s" and into the folder "%(gnu)s" in your "%(doc)s" folder.\nMay I move the files automatically for you now?' % {
                     'doc':  os.path.split(os.path.split(filesystem.user_data())[0])[1],
                     'gnu':  os.path.join(filesystem.appname, 'lessonfiles'),
@@ -894,8 +472,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                     else:
                         gu.dialog_ok(_("Files copied. The old files has been left behind. Please delete them when you have verified that all files was copied correctly."))
 
-                if splash_win:
-                    splash_win.show()
+                if solfege.splash_win:
+                    solfege.splash_win.show()
         # MIGRATION 3.9.3 when we added langenviron.bat and in 3.11
         # we migrated to langenviron.txt because we does not use cmd.exe
         if sys.platform == 'win32' and winlang.win32_get_langenviron() != self.get_string('app/lc_messages'):
@@ -934,7 +512,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             else:
                 self.get_view().g_notebook.set_current_page(0)
         self.set_title("Solfege - " + self.get_view().m_t.m_P.header.title)
-    def display_docfile(self, fn, anchor):
+    def display_docfile(self, fn):
         """
         Display the HTML file named by fn in the help browser window.
         """
@@ -1008,12 +586,15 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def on_search_all_exercises(self, widget=None):
         self.set_title("GNU Solfege")
         if not self.show_view('searchview'):
-            self.add_view(SearchView(), 'searchview')
+            self.add_view(SearchView(u'Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.'), 'searchview')
     def display_frontpage(self, data=None, show_topics=False):
         """
         Display the front page of the data  in solfege.app.m_frontpage_data
         """
-        self.set_title("GNU Solfege")
+        if solfege.app.m_options.profile:
+            self.set_title("GNU Solfege - %s" % solfege.app.m_options.profile)
+        else:
+            self.set_title("GNU Solfege")
         if not self.show_view('frontpage'):
             self.add_view(FrontPage(), 'frontpage')
         self.get_view().g_searchbox.show()
@@ -1030,12 +611,36 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         m = __import__("solfege.exercises.%s" % teacher.m_exname, fromlist=("solfege.exercises.%s" % teacher.m_exname,), level=0)
         self.add_view(m.Gui(teacher), teacher.m_exname)
     def on_key_press_event(self, widget, event):
-        self.get_view().on_key_press_event(widget, event)
+        try:
+            view = self.get_view()
+        except KeyError:
+            return
+        view.on_key_press_event(widget, event)
+    def open_profile_manager(self, widget=None):
+        p = ChangeProfileDialog(solfege.app.m_options.profile)
+        if p.run() == gtk.RESPONSE_ACCEPT:
+            prof = p.get_profile()
+        else:
+            # The user presses cancel. This will use the same profile as
+            # before, but if the user has renamed the active profile, then
+            # we need to use the new name.
+            prof = p.m_default_profile
+
+        solfege.app.reset_exercise()
+        solfege.app.m_options.profile = prof
+        solfege.db.conn.commit()
+        solfege.db.conn.close()
+        solfege.db = statistics.DB(None, profile=prof)
+        cfg.set_string("app/last_profile", prof)
+        self.display_frontpage()
+        p.destroy()
     def open_preferences_window(self, widget=None):
         if not self.g_config_window:
             self.g_config_window = ConfigWindow()
             self.g_config_window.show()
         else:
+            self.g_config_window.update_old_statistics_info()
+            self.g_config_window.update_statistics_info()
             self.g_config_window.show()
     def quit_program(self, *w):
         can_quit = True
@@ -1084,59 +689,4 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         dlg = PractiseSheetDialog()
         dlg.show_all()
 
-import locale
-locale.setlocale(locale.LC_NUMERIC, "C")
-
-# check_rcfile has to be called before and
-# functions that use the cfg module.
-application.check_rcfile()
-
-cfg.set_bool('config/no_random', bool(options.no_random))
-
-lessonfile.infocache = lessonfile.InfoCache()
-
-if splash_win:
-    if not os.path.exists(statistics.DB.get_statistics_filename()):
-        splash_win.show_progress(_("Importing old statistics"))
-def f(s):
-    if splash_win:
-        splash_win.show_progress(s)
-solfege.db = statistics.DB(f)
-
-if splash_win:
-    splash_win.show_progress(_("Creating application window"))
-
-def start_app(datadir):
-    global splash_win
-    solfege.app = application.SolfegeApp(options)
-    solfege.win = w = MainWin(options, datadir)
-    solfege.app.setup_sound()
-    w.show()
-    w.post_constructor()
-    if splash_win:
-        splash_win.destroy()
-        splash_win = None
-
-    def ef(t, value, traceback):
-        if options.debug:
-            msg = "ehooked:" + str(value)
-        else:
-            msg = str(value)
-        if issubclass(t, lessonfile.LessonfileException):
-            w.display_error_message(msg, str(t))
-        elif issubclass(t, osutils.ExecutableDoesNotExist):
-            if len(value.args) > 1:
-                w.display_error_message2(value.args[0], "\n".join(value.args[1:]))
-            else:
-                w.display_error_message(msg, str(t))
-        else:
-            sys.__excepthook__(t, msg, traceback)
-    if not options.disable_exception_handler:
-        sys.excepthook = ef
-    print time.time() - start_time
-    # We parse all lesson files when we are idle to save a half a
-    # second the first time the user searches all lesson files using
-    # Ctrl-F.
-    lessonfile.infocache.parse_all_files(True)
-    gtk.main()
 
